/***************************************************************************
  mctcheck.c  Version 1.0.2 
  -------------------
     copyright            : (C) 2007 by 
                           Sagem-Monetel GmbH
                           Konrad-Zuse-Ring 1
                           24220 Flintbek
                           Germany
                           Bernd Nordquist
    e-mail                : bnordquist@sagem-monetel.de
    Last change:         : 15.5.2001
    Last change:         : 16.8.2008


 ***************************************************************************/

/***************************************************************************
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/****************************************************************
*
*   Einfaches Testprogramm fuer die distributierte 
*   CT-API  libCTORGT1.so.1.0.0 (Shared Library)
*   
*   Dieses Programm benoetigt die Shared Library
*   libCTORGT1.so.1.0.0
*	Es muss gegen diese gelinkt werden (siehe 
*	Shell-Skript "compile_mctcheck"
*	
*   Die Shared Library libCTORGT1.so.1.0.0 
*	ist die CT-API zum Programmieren eines 
*	CardTerminals "ORGA MCT5000/6000 Serie".
*	Sie ist keine freie Software, sondern nur als 
*	Binary kostenlos verfuegbar.
*	
*	 
*  Aufgaben:
*			
*  1. Freie und vorhandende bzw. konfigurierte Schnittstellen 
*     (ttySx, ttyACMx, ttyUSBx) im System suchen.
*			
*  2. Nachsehen, ob die Schnittstellen nicht gesperrt sind
*    /var/lock/LCK..ttySx.
*
*  3. Freie Schnittstellen nach angeschlossenem MKT absuchen.
*		
*  4. BCS Kommandos "RESET_CT" und "GET_STATUS" an angeschlossenen
*     MCTs senden.
*
****************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <stdio.h>
#include <stdlib.h>

//#include "ctapi.h"
#include "../../htsi_src/h/ctapi.h"


#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#define CTN_1   1   // Card Terminal Number one
#define CTN_2   2
#define CTN_3   3   // Card Terminal Number 3
#define CTN_4   4

#define DAD_CT   	1 // destination address is card terminal
#define DAD_ICC_1 0 // destination address is ICC 1
#define DAD_ICC_2 2 // destination address is ICC 2
#define SAD_HOST 2 // Host is sad (source address)

#define MAX_RESPONSE 1024

#define DEV_FILE_PN_1_DFLT  "/dev/ttyS0"
#define DEV_FILE_PN_2_DFLT  "/dev/ttyS1"
#define DEV_FILE_PN_3_DFLT  "/dev/ttyS2"
#define DEV_FILE_PN_4_DFLT  "/dev/ttyS3"

#define LOCK_FILE_PN_1_DFLT "/var/lock/LCK..ttyS0"
#define LOCK_FILE_PN_2_DFLT "/var/lock/LCK..ttyS1"
#define LOCK_FILE_PN_3_DFLT "/var/lock/LCK..ttyS2"
#define LOCK_FILE_PN_4_DFLT "/var/lock/LCK..ttyS3"

#define MAX_DEV_FILE_NAME    30
#define MAX_LOCK_FILE_NAME   40

/**********************************************
 * Global device file names.
 * Default is: 
 * /dev/ttyS0 - /dev/ttyS3
 **********************************************/
static char dev_file_name  [4][MAX_DEV_FILE_NAME];
/**********************************************
 * Global lock file names.
 * Default is: 
 * /var/lock/LCK..ttyS0 - /var/lock/LCK..ttyS3
 **********************************************/
static char lock_file_name [4][MAX_LOCK_FILE_NAME];


// The constant and absolut file path
static const char * conf_file = "/usr/local/etc/ctorg.conf";
//static const char * lokal_conf_file = "./ctorg.conf";

int CtFunktionReturnString(int ret_value, char** ret_string);
int InitCt(unsigned short ctn, unsigned short pn);
int CtPortString(int port_number, char** port_string);
int CloseCt(unsigned short ctn);
int ResetCt(unsigned char ctn, unsigned char* response);
int GetStatus(unsigned char ctn, char* response);
int print_response(unsigned char dad,unsigned char sad,
	int lenr, unsigned char *rsp);
int CheckForSerialPort(char* port);
int CheckForLockFile(char* lockfile);
int CheckForCt(unsigned char ctn,unsigned short pn);
int make_null(int* serial_port, int* lock_file, int* ct_at_port);

int print_status(int *serial_port, int *lock_file, int *ct_at_port);

static int  read_conf_file          ( void );
static void init_file_names         ( void );
static int  search_for_dev_file_name( char *line, int pn );
static void set_lck_file_name       ( char * dev_file, char * lock_file );

/***************************************************
*
*  main()
*
*
***************************************************/
int main(int argc, char *argv[])
{

	int ret;
	unsigned char response[MAX_RESPONSE]; // array for response
	int serial_port [5];  // 0 == not exists, 1 == exists
	int lock_file[5];     // 0 == not exists, 1 == exists
	int ct_at_port[5];     // 0 == not exists, 1 == exists
	int ct_speed;

	make_null( serial_port,  lock_file,  ct_at_port);

// Print Program version
	printf("\n mctcheck version 1.0.2");

	
	printf("\n\n");
	printf("1. Checking if %s exists:\n", conf_file);
	printf("   --------------------------------------------");
	if( read_conf_file() == 0)
	{
		printf("\n   Config file %s exists", conf_file);
		printf("\n   Use the following device files:");
	}
	else
	{
		printf("\n   Config file %s not exists", conf_file);
		printf("\n   Use default device files:");
	}
	printf("\n   pn=1 => %s", &dev_file_name[0][0]);
	printf("\n   pn=2 => %s", &dev_file_name[1][0]);
	printf("\n   pn=3 => %s", &dev_file_name[2][0]);
	printf("\n   pn=4 => %s", &dev_file_name[3][0]);
	
	
		// Check what ports do exists on the system
	printf("\n\n");
	printf("2. Checking if ports are exists on the system:\n");
	printf("   --------------------------------------------");


	serial_port[PORT_COM1] = CheckForSerialPort(&dev_file_name[0][0]);
	if(serial_port[PORT_COM1]==1)
		printf("\n   Port %s existing", &dev_file_name[0][0]);

	serial_port[PORT_COM2] = CheckForSerialPort(&dev_file_name[1][0]);
	if(serial_port[PORT_COM2]==1)
		printf("\n   Port %s existing", &dev_file_name[1][0]);

	serial_port[PORT_COM3] = CheckForSerialPort(&dev_file_name[2][0]);
	if(serial_port[PORT_COM3]==1)
		printf("\n   Port %s existing", &dev_file_name[2][0]);

	serial_port[PORT_COM4] = CheckForSerialPort(&dev_file_name[3][0]);
	if(serial_port[PORT_COM4]==1)
		printf("\n   Port %s existing", &dev_file_name[3][0]);

// Check if the existing ports are locked
	printf("\n\n3. Checking if the existing ports are locked:\n");
	printf("   --------------------------------------------");

	if(serial_port[PORT_COM1] == 1)
	{
		lock_file[PORT_COM1] = CheckForLockFile(&lock_file_name[0][0]);
		if(lock_file[PORT_COM1] == 0)
			printf("\n   Port %s is NOT locked", &dev_file_name[0][0]);
		if(lock_file[PORT_COM1] == 1)
			printf("\n   Port %s is locked", &dev_file_name[0][0]);
	}

	if(serial_port[PORT_COM2] == 1)
	{
		lock_file[PORT_COM2] = CheckForLockFile(&lock_file_name[1][0]);
		if(lock_file[PORT_COM2] == 0)
			printf("\n   Port %s is NOT locked", &dev_file_name[1][0]);
		if(lock_file[PORT_COM2] == 1)
			printf("\n   Port %s is locked", &dev_file_name[1][0]);
	}
	if(serial_port[PORT_COM3] == 1)
	{
		lock_file[PORT_COM3] = CheckForLockFile(&lock_file_name[2][0]);
		if(lock_file[PORT_COM3] == 0)
			printf("\n   Port %s is NOT locked", &dev_file_name[2][0]);
		if(lock_file[PORT_COM3] == 1)
			printf("\n   Port %s is locked", &dev_file_name[2][0]);
	}
	if(serial_port[PORT_COM4] == 1)
	{
		lock_file[PORT_COM4] = CheckForLockFile(&lock_file_name[3][0]);
		if(lock_file[PORT_COM4] == 0)
			printf("\n   Port %s is NOT locked", &dev_file_name[3][0]);
		if(lock_file[PORT_COM3] == 1)
			printf("\n   Port %s is locked", &dev_file_name[3][0]);
	}

// Check if there is a CT at the existing and not locked ports
	printf("\n\n4. Checking if there is a CT at the existing and not locked ports:\n");
	printf("   --------------------------------------------------------------");

	if(lock_file[PORT_COM1] == 0)
	{
		printf("\n   +++ Searching at %s... Can take a few seconds...\n", &dev_file_name[0][0]);
		ct_at_port[PORT_COM1] = CheckForCt(CTN_1, PORT_COM1);
		if(ct_at_port[PORT_COM1] == 1)
		{	
			CT_getbaud(PORT_COM1, &ct_speed);
			printf("\n   CT at port %s available. Speed is: %d bps", &dev_file_name[0][0], ct_speed);
		}
		if(ct_at_port[PORT_COM1] == 0)
			printf("\n   CT at port %s NOT available", &dev_file_name[0][0]);
	}

	if(lock_file[PORT_COM2] == 0)
	{
		printf("\n\n   +++ Searching at %s... Can take a few seconds...\n", &dev_file_name[1][0]);
		ct_at_port[PORT_COM2] = CheckForCt(CTN_2, PORT_COM2);
		if(ct_at_port[PORT_COM2] == 1)
		{
			CT_getbaud(PORT_COM2, &ct_speed);
			printf("\n   CT at port %s available. Speed is: %d bps", &dev_file_name[1][0], ct_speed);
		}
		if(ct_at_port[PORT_COM2] == 0)
			printf("\n   CT at port %s NOT available", &dev_file_name[1][0]);
	}

	if(lock_file[PORT_COM3] == 0)
	{
		printf("\n\n   +++ Searching at %s... Can take a few seconds...\n", &dev_file_name[2][0]);
		ct_at_port[PORT_COM3] = CheckForCt(CTN_3, PORT_COM3);
		if(ct_at_port[PORT_COM3] == 1)
		{	
			CT_getbaud(PORT_COM3, &ct_speed);
			printf("\n   CT at port %s available. Speed is: %d bps", &dev_file_name[2][0], ct_speed);
		}
		if(ct_at_port[PORT_COM3] == 0)
			printf("\n   CT at port %s NOT available", &dev_file_name[2][0]);
	}

	if(lock_file[PORT_COM4] == 0)
	{
		printf("\n\n   +++ Searching at %s...Can take a few seconds...\n", &dev_file_name[3][0]);
		ct_at_port[PORT_COM4] = CheckForCt(CTN_4, PORT_COM4);
		if(ct_at_port[PORT_COM4] == 1)
		{
			CT_getbaud(PORT_COM4, &ct_speed);
			printf("\n   CT at port %s available. Speed is: %d bps", &dev_file_name[3][0], ct_speed);
		}
		if(ct_at_port[PORT_COM4] == 0)
			printf("\n   CT at port %s NOT available", &dev_file_name[3][0]);
	}


// Sending Commands to the existing Card Terminals
	if( ct_at_port[PORT_COM1]==1 || ct_at_port[PORT_COM2]==1 || \
		ct_at_port[PORT_COM3]==1 || ct_at_port[PORT_COM4]==1 )
	{
		printf("\n\n4. Sending Commands to the existing Card Terminals\n");
		printf("   -------------------------------------------------");

		if(ct_at_port[PORT_COM1] == 1)
		{
			printf("\n");
			ret = InitCt(CTN_1, PORT_COM1);
			if(ret == 0)
			{
				printf("\n");
				ResetCt(CTN_1, response);
				printf("\n\n");

				GetStatus(CTN_1, (char*)response);
				printf("\n");
				CloseCt(CTN_1);
			}
		}


		if(ct_at_port[PORT_COM2] == 1)
		{
			printf("\n");
			ret = InitCt(CTN_2, PORT_COM2);
			if(ret == 0)
			{
				printf("\n");
				ResetCt(CTN_2, response);
				printf("\n");

				GetStatus(CTN_2, (char*)response);
				printf("\n");
				CloseCt(CTN_2);
			}
		}

		if(ct_at_port[PORT_COM3] == 1)
		{
			printf("\n");
			ret = InitCt(CTN_3, PORT_COM3);
			if(ret == 0)
			{
				printf("\n");
				ResetCt(CTN_3, response);
				printf("\n");
				GetStatus(CTN_3, (char*)response);
				printf("\n");
				CloseCt(CTN_3);
			}
		}

		if(ct_at_port[PORT_COM4] == 1)
		{

			printf("\n");
			ret = InitCt(CTN_4, PORT_COM4);
			if(ret == 0)
			{
				printf("\n");
				ResetCt(CTN_4, response);
				printf("\n");
				GetStatus(CTN_4, (char*)response);
				printf("\n");
				CloseCt(CTN_4);
			}
		}

	}
	else
		printf("\n\n   No CT connected!\n\n");

	return EXIT_SUCCESS;
}



/***************************************************
*
*  ResetCt()
*
*
***************************************************/
int ResetCt(unsigned char ctn, unsigned char* response)
{

	char RESET_CT[5] = { 0x20,0x11, 0x00, 0x00, 0x00 };
/* SW1 SW2 = 9000, oder 6400 (not seccessful)
*/
// Information:
//--------------
//char* command;        // pointer for command
//unsigned short lenc;  // length of command
//unsigned short lenr;  // length of response
//unsigned short dad;  // destination address
//unsigned short sad;  // source address
////unsigned short ctn;  // number of card terminal
//unsigned short pn;  // Port number; see header of the manufacturer
///*
//CT_data(unsigned short ctn, unsigned char *dad, unsigned char *sad,
//              unsigned short lenc, unsigned char *command,
//              unsigned short *lenr, unsigned char *response );
//
//#define DAD_CT   	1 // destination address is card terminal
//#define DAD_ICC_1 0 // destination address is ICC 1
//#define DAD_ICC_2 2 // destination address is ICC 2
//#define SAD_HOST 2 // Host is sad (source address)

	int ret_value;
	int ret;
	char* ret_string;

	char* command = RESET_CT;        // pointer for command
	unsigned short lenc = 5;  // length of command
	unsigned short lenr = MAX_RESPONSE;  // length of response
	unsigned char dad  = DAD_CT;  // destination address
	unsigned char sad  = SAD_HOST;  // source address

	printf("   >>> Sending RESET_CT to CT %d :: ", ctn);

//ret_value = CT_data( ctn, &dad, &sad, lenc, command, &lenr, response );
	ret_value = CTDEORGT1_data(ctn, &dad, &sad, lenc, (unsigned char*)command, &lenr, response);


	ret = CtFunktionReturnString(ret_value, &ret_string);
	if(ret == 0)
		printf(" [%d] == %s\n", ret_value, ret_string);

	else
		printf("Unknown response\n");

	print_response(dad,sad,lenr,response);

	return 0;
}



/***************************************************
*
*  GetStatus()
*
*
***************************************************/
int GetStatus(unsigned char ctn, char* response)
{

 // pointer for command
	char GET_STATUS[5] = { 0x20, 0x13, 0x00, 0x46, 0x00 };

	int ret_value;
	int ret;
	char* ret_string;


	unsigned short lenc = 5;  // length of command
	unsigned short lenr = MAX_RESPONSE;  // length of response
	unsigned char dad  = DAD_CT;  // destination address
	unsigned char sad  = SAD_HOST;  // source address

	printf("   >>> Sending GET_STATUS to CT %d :: ", ctn);

//ret_value = CT_data( ctn, &dad, &sad, lenc, GET_STATUS, &lenr, response );
	ret_value = CTDEORGT1_data(ctn, &dad, &sad, lenc, (unsigned char*)GET_STATUS, &lenr, (unsigned char*)response);

	ret = CtFunktionReturnString(ret_value, &ret_string);
	if(ret == 0)
		printf(" [%d] == %s\n", ret_value, ret_string);
	else
		printf("Unknown response\n");

	print_response(dad,sad,lenr,(unsigned char*)response);

	return 0;
}

/***************************************************
*
*  InitCt()
*
*
***************************************************/
int InitCt(unsigned short ctn, unsigned short pn)
{

	int ret_value;
	int ret;
	char* ret_string;
	char* port_string;

	CtPortString(pn, &port_string);
	printf("  *** Initializing CT %d at %s :: ", ctn, port_string);

//ret_value = CT_init( ctn, pn);
	ret_value = CTDEORGT1_init(ctn, pn);

	ret = CtFunktionReturnString(ret_value, &ret_string);
	if(ret == 0)
		printf(" [%d] == %s\n", ret_value, ret_string);
	else
		printf("Unknown response\n");

	if(ret_value != 0)
		return -1;

	return 0;
}

/***************************************************
*
*  CloseCt()
*
*
***************************************************/
int CloseCt(unsigned short ctn)
{
	int ret_value;
	int ret;
	char* ret_string;
	printf("   *** Closing CT %d  :: ", ctn);

//ret_value = CT_close( ctn);
	ret_value = CTDEORGT1_close(ctn);


	ret = CtFunktionReturnString(ret_value, &ret_string);
	if(ret == 0)
		printf(" [%d] == %s\n", ret_value, ret_string);
	else
		printf("Unknown response\n");

//printf("\n");

	return 0;
}



/***************************************************
*
*  CtFunktionReturnString
*
*
***************************************************/
int CtFunktionReturnString(int ret_value, char** ret_string)
{

	static char ok[] = "OK";   								 // 0
	static char err_invalid[] = "ERR_INVALID"; // -1
	static char err_ct[] = "ERR_CT";           // -8
	static char err_trans[] = "ERR_TRANS";     // -10
	static char err_memory[] = "ERR_MEMORY" ;  // -11
	static char err_host[] = "ERR_HOST" ;      // -127
	static char err_htsi[] = "ERR_HTSI" ;      // -128


	switch (ret_value)
	{
		case 0:	*ret_string = ok;
			break;

		case -1: *ret_string = err_invalid;
			break;

		case -8:  *ret_string = err_ct;
			break;

		case -10: *ret_string = err_trans;
			break;

		case -11: *ret_string = err_memory;
			break;

		case -127: *ret_string = err_host;
			break;

		case -128: *ret_string = err_htsi;
			break;

		default: return -1;

	}

	return 0;
}

/***************************************************
*
*  CtPortString
*
*
***************************************************/
int CtPortString(int port_number, char** port_string)
{
	//static char port1[] = "/dev/ttyS0";  								
	//static char port2[] = "/dev/ttyS1";  								
	//static char port3[] = "/dev/ttyS2";  								
	//static char port4[] = "/dev/ttyS3";  								

	switch (port_number)
	{
		case PORT_COM1:	*port_string = &dev_file_name[0][0];
			break;

		case PORT_COM2: *port_string = &dev_file_name[1][0];
			break;

		case PORT_COM3: *port_string = &dev_file_name[2][0];
			break;

		case PORT_COM4: *port_string = &dev_file_name[3][0];
			break;


		default: return -1;

	}

	return 0;
}


/*****************************************************
*
*  print_response()
*
*
*****************************************************/
int print_response(unsigned char dad,unsigned char sad,
	int lenr, unsigned char *rsp)
{
	int k, m;
	int counter;

	printf("   <<< Response from Card Terminal:");
	printf("\n   ==> dad = %d   sad = %d  lenr = %d", dad, sad, lenr);
	printf("\n   ==> Response APDU (HEX) = ");

	m = 0;
	counter =0;
	for(k=0;k<lenr; k++)
	{
		if(m == 10)
		{
			printf("\n                             ");
			m=0;
		}
		m++;
		printf(" [%X]", rsp[k]);
		counter++;
	}
	printf("\n");
	printf("   ==> Response APDU (TEXT)=  ");
	m = 0;
	counter =0;
	for(k=0;k<lenr-2; k++)
	{
		if(m == 20)
		{
			printf("\n                             ");
			m=0;
		}
		m++;
		printf("%c", rsp[k]);
		counter++;
	}
	printf("\n");

	return 0;
}

/***************************************************
*
*  CheckForLockFile()
*
*
***************************************************/
int CheckForLockFile(char* lockfile)
{
	int ret;
	int ret_value;

	FILE* fd1;
	char string_pid[20];
	int i;
	char c;
	char filename[20] = "/proc/";

// 1. look for the lockfile
	ret =  access(lockfile, 00);  /* 00 fuer "Datei existiert" */

	if(ret == 0)   // lockfile exists
	{
// 2. read PID from lockfile
		printf("\n!!!!! %s", lockfile);
		fd1 =	fopen(lockfile, "r");

		i=0;
		while( (c = fgetc(fd1)) != EOF)  // read PID
		{

			if((c=='0'||c=='1'||c=='2'||c=='3'||c=='4'||c=='5'||c=='6'||c=='7'||c=='8'||c=='9'))
			{
				string_pid[i] =c;
				i++;

			}
			if(i==20){
				 // this can`t be a PID, to long!
				printf("\n!!!!! The file %s seems to be a bad lockfile!",lockfile);
				printf("\n!!!!! PID is longer then 20 Bytes.");
				fclose(fd1);
				return -1;
			}
		}
		string_pid[i] = '\0';


// 3.  look under /proc/* if the Process with this PID already running
		sprintf(&filename[6],"%s", string_pid);
		printf("  ==>%s", filename);
		ret =  access(filename, 00);  /* 00 fuer "Datei existiert" */

		if( ret == -1) // Process doesn't exists!
		{
       // delete the lockfile
			ret_value = remove(lockfile);
			fclose(fd1);
			printf("\n Process %s isn't existing no more", string_pid);
			return 0; // now port is free
		}	
  // else serial port is locked
		else{
			fclose(fd1);		
			return 1;
		}
	}

	if( ret == -1)  // no lockfile exists (not locked)
		return 0;
	return 0;
}

/***************************************************
*
*  CheckForSerialPort()
*
*
***************************************************/
int CheckForSerialPort(char* port)
{
	int fd0;
	int t0;
	int fkt_rsp;


/*******************************
* open serial port ttySx
*******************************/
	fd0= open (port , O_RDONLY | O_NOCTTY  );
	if( fd0 == -1)
	{
		fkt_rsp = close(fd0);
		return -1;
	}	

	t0 = isatty(fd0);
	if(t0 == 1)
	{
		fkt_rsp = close(fd0);
		return 1;
	}

	fkt_rsp = close(fd0);
	return 0;

}



/***************************************************
*
*  CheckForCt()
*
*
***************************************************/
int CheckForCt(unsigned char ctn,unsigned short pn)
{

	int ret_value;

//ret_value = CT_init( ctn, pn);
	ret_value = CTDEORGT1_init(ctn, pn);

	if(ret_value != 0)
	{
//  printf("\n I am in Funktion CheckForCT ==> CT_init ret_value = %d", ret_value);
		;
		if(ret_value == -10) // ERR_TRANS
			return 0; // no CT available
		else
			return 1;  //
	}
	else
	{

//printf("\n I am in Funktion CheckForCT ==> CT_init ret_value = %d\n", ret_value);
		CloseCt(ctn);
		return 1;
	}

}


/***************************************************
*
*  make_null()
*
*
***************************************************/
int make_null(int *serial_port, int *lock_file, int *ct_at_port)
{
	int i;

	for(i=0;i<5;i++)
		serial_port[i] = 0;

	for(i=0;i<5;i++)
		lock_file[i] = 1;

	for(i=0;i<5;i++)
		ct_at_port[i] = 0;

	return 0;
}


/***************************************************
*
*  print_status()
*   for debugging
*
***************************************************/
int print_status(int *serial_port, int *lock_file, int *ct_at_port)
{
	int i;

	for(i=0;i<5;i++)
		printf("\n serial_port %d = %d", i, serial_port[i]);

	for(i=0;i<5;i++)
		printf("\n lock_file %d = %d", i, lock_file[i]);

	for(i=0;i<5;i++)
		printf("\n ct_at_port %d = %d", i, ct_at_port[i]);

	return 0;
}

static int read_conf_file(void)
{

	FILE * fd;
	int max_line = 98;
	char line[100];
	int ret;
	int line_count = 0;
	
	int i;
	
	
	
	init_file_names();
	
   //sehen, ob Datei vorhanden mit access()
   // wenn da, dann oeffnen, auslesen, schliessen

    /* check if file exists */
	ret =  access(conf_file, 00);  /* 00 fuer "Datei existiert" */
	if( ret == 0) // file exists
	{
       //printf("file exists\n");
		fd = fopen( conf_file, "r"); // open for reading
		if(fd == NULL) 
		{
         //printf(" open error\n");
			return -1;  // open error
		}

       // parse the complete file
		while( fgets(line, max_line, fd) != NULL)
		{
			line_count++;
          // kommentar herausfiltern
			for(i=0; i< max_line; i++)
			{  
				if(line[i] == '\0')
					break;  // no # found
				else if(line[i] == '#') // rest ist kommentar
				{
					line[i] = '\0';
					break;
				}
			}
			//printf("line :: %s", line);

			for( i=1; i<5 ; i++)
			{
				ret = search_for_dev_file_name(line, i);
			}
			
		} // while  
		
		ret = fclose(fd);
		if(ret == EOF) return -1; // close error
	}
	else  // if (ret == 0) access
		return -2; // file does not exist

	return 0;
}

/**********************************************************
*
*  Name:  search_for_dev_file_name()
*
*  Purpose: Analyse the line parameter if the key
*           "device_file_name_for_pn_<pn>=" is present. If so
*           the value part will be stored in the global
*           array "dev_file_name[pn-1][]" and the lock file
*           name will be set accordingly in the global
*           "lock_file_name[pn-1][]" array. 
* 
*  Parameter:     line - one row from the "ctorg.conf" file
* 
*                 pn   - Port number > 0
* 
*  Return value : 1 key value found
*                 0 key value not found
*                -1 in case of an error
************************************************************/

static int search_for_dev_file_name(char *line, int pn)
{
	char dev_file[27]; // "device_file_name_for_pn_x=";
	int  j;
	char * p_str;

	if( pn <= 0 )
	{
		return -1;
	}
	
	snprintf(dev_file, 27, "device_file_name_for_pn_%d=", pn);
	
	p_str = strstr(line, dev_file);  // sucht nach device_file_name_for_pn_x= (leerzeichen werden eleminiert)
	if(p_str != NULL)
	{
		//printf("search_for_dev_file_name: %s\n", p_str);
		for(j=strlen(dev_file) ;  ; j++)
		{
              // look for end of device file name i.e. 192.168.1.005 
			if(p_str[j] == 0x20 || p_str[j] == '\n' || p_str[j] == '\0')
			{
				p_str[j] = '\0';
				strncpy(&dev_file_name[pn-1][0], p_str + strlen(dev_file), MAX_DEV_FILE_NAME-1);
				set_lck_file_name( &dev_file_name[pn-1][0], &lock_file_name[pn-1][0]);
				return 1;
			}   
		}
		return -1;  // error
	} 
	
	return 0;

}

/**********************************************************
*
*  Name:  set_lck_file_name()
*
*  Purpose: Sets the lock file name for the passed device
*           file name parameter. E.g. if the device file 
*           parameter is "/dev/ttyACM0" the lock file 
*           will be set to "/var/lock/LCK..ttyACM0"
* 
*  Parameter:     dev_file  - device file name (input)
* 
*                 lock_file - lock file name (output)
* 
*  Return value : -
* 
************************************************************/

static void set_lck_file_name(char * dev_file, char * lock_file)
{
	char *lock_file_pre = "/var/lock/LCK..";
	
	
	if( strstr(dev_file, "/dev/") != NULL )
	{
		strcpy( lock_file, lock_file_pre);
		strncat(lock_file, dev_file + 5, MAX_LOCK_FILE_NAME-1);
		//printf("set_lck_file_name: lock_file = %s\n", lock_file);
	}
	
}

/**********************************************************
*
*  Name:  init_file_names()
*
*  Purpose: Initialise the default device file and lock
*           file names.
*           Device files:
*           /dev/ttyS0 - /dev/ttyS3
*           Lock files:
*           /var/lock/LCK..ttyS0 - /var/lock/LCK..ttyS3
* 
*  Parameter:     -
* 
*  Return value : -
* 
************************************************************/

static void init_file_names( void )
{
	
	strcpy( & dev_file_name[0][0], DEV_FILE_PN_1_DFLT );
	strcpy( & dev_file_name[1][0], DEV_FILE_PN_2_DFLT );
	strcpy( & dev_file_name[2][0], DEV_FILE_PN_3_DFLT );
	strcpy( & dev_file_name[3][0], DEV_FILE_PN_4_DFLT );

	strcpy( & lock_file_name[0][0], LOCK_FILE_PN_1_DFLT );
	strcpy( & lock_file_name[1][0], LOCK_FILE_PN_2_DFLT );
	strcpy( & lock_file_name[2][0], LOCK_FILE_PN_3_DFLT );
	strcpy( & lock_file_name[3][0], LOCK_FILE_PN_4_DFLT );
	
}
